@@ -1,6 +1,9 @@
 """
-✏️safer: a safer file opener ✏️
--------------------------------
+🧿 safer: safer writing in Python 🧿
+--------------------------------------
+
+.. image:: https://raw.githubusercontent.com/rec/safer/master/safer.png
+   :alt: safer logo
 
 .. doks-shields::
 
@@ -10,42 +13,49 @@
 No more partial writes or corruption! Wraps file streams, sockets or
 any callable.
 
-Install ``safer`` from the command line with `pip
-<https://pypi.org/project/pip>`_: ``pip install safer``.
+Use `pip <https://pypi.org/project/pip>`_ to install `safer` from the command
+line: `pip install safer`.
 
-Tested on Python 3.4 and 3.8 - Python 2.7 version
-is here <https://github.com/rec/safer/tree/v2.0.5>`_.
+Tested on Python 3.4 - 3.9.  An old Python 2.7 version
+is `here <https://github.com/rec/safer/tree/v2.0.5>`_.
 
 See the Medium article `here. <https://medium.com/@TomSwirly/\
 %EF%B8%8F-safer-a-safer-file-writer-%EF%B8%8F-5fe267dbe3f5>`_
 
 -------
 
-``safer`` helps prevent programmer error from corrupting files, socket
-connections, or generalized streams
+`safer` helps prevent programmer error from corrupting files, socket
+connections, or generalized streams by writing a whole file or nothing.
 
 It does not prevent concurrent modification of files from other threads or
 processes: if you need atomic file writing, see
 https://pypi.org/project/atomicwrites/
 
-* ``safer.writer()`` wraps an existing writer, socket or stream and writes a
+It also has a useful `dry_run` setting to let you test your code without
+actually overwriting the target file.
+
+* `safer.writer()` wraps an existing writer, socket or stream and writes a
   whole response or nothing
 
-* ``safer.open()`` is a drop-in replacement for built-in ``open`` that
+* `safer.open()` is a drop-in replacement for built-in `open` that
   writes a whole file or nothing
 
-* ``safer.closer()`` returns a stream like from ``safer.write()`` that also
+* `safer.closer()` returns a stream like from `safer.write()` that also
   closes the underlying stream or callable when it closes.
 
-* ``safer.printer()`` is ``safer.open()`` except that it yields a
+* `safer.dump()` is like a safer `json.dump()` which can be used for any
+  serialization protocol, including Yaml and Toml, and also allows you to
+  write to file streams or any other callable.
+
+* `safer.printer()` is `safer.open()` except that it yields a
   a function that prints to the stream.
 
-By default, ``safer`` buffers the written data in memory in a ``io.StringIO``
-or ``io.BytesIO``.
+By default, `safer` buffers the written data in memory in a `io.StringIO`
+or `io.BytesIO`.
 
-For very large files, ``safer.open()`` has a ``temp_file`` argument which
+For very large files, `safer.open()` has a `temp_file` argument which
 writes the data to a temporary file on disk, which is moved over using
-``os.rename`` if the operation completes successfully.  This functionality
+`os.rename` if the operation completes successfully.  This functionality
 does not work on Windows.  (In fact, it's unclear if any of this works on
 Windows, but that certainly won't.  Windows developer solicted!)
 
@@ -54,10 +64,10 @@ Windows, but that certainly won't.  Windows developer solicted!)
 EXAMPLES
 =========
 
-``safer.writer()``
+`safer.writer()`
 ~~~~~~~~~~~~~~~~~~~
 
-``safer.writer()`` wraps an existing stream - a writer, socket, or callback -
+`safer.writer()` wraps an existing stream - a writer, socket, or callback -
 in a temporary stream which is only copied to the target stream at close() and
 only if no exception was raised.
 
@@ -85,11 +95,11 @@ EXAMPLE
      except:
         write_error(sock)  # Nothing has been written
 
-``safer.open()``
+`safer.open()`
 ~~~~~~~~~~~~~~~~~
 
 Writes a whole file or nothing. It's a drop-in replacement for built-in
-``open()`` except that ``safer.open()`` leaves the original file unchanged on
+`open()` except that `safer.open()` leaves the original file unchanged on
 failure.
 
 EXAMPLE
@@ -108,25 +118,63 @@ EXAMPLE
         # If an exception is raised, the file is unchanged.
 
 
-``safer.open(filename)`` returns a file stream ``fp`` like ``open(filename)``
-would, except that ``fp`` writes to memory stream or a temporary file in the
+`safer.open(filename)` returns a file stream `fp` like `open(filename)`
+would, except that `fp` writes to memory stream or a temporary file in the
 same directory.
 
-If ``fp`` is used as a context manager and an exception is raised, then the
-property ``fp.safer_failed`` on the stream is automatically set to ``True``.
+If `fp` is used as a context manager and an exception is raised, then the
+property `fp.safer_failed` on the stream is automatically set to `True`.
 
-And when ``fp.close()`` is called, the cached data is stored in ``filename`` -
-*unless* ``fp.safer_failed`` is true.
+And when `fp.close()` is called, the cached data is stored in `filename` -
+*unless* `fp.safer_failed` is true.
 
 ------------------------------------
 
-``safer.printer()``
+`safer.dump()`
+~~~~~~~~~~~~~~~~~
+
+Serializes a whole file or nothing. It's a drop-in replacement for
+`json.dump()` except:
+
+* `safer.dump()` leaves the original file unchanged on
+* It takes a filename in preference to an open file stream
+* But it handles files, socket streams, or any callable
+
+EXAMPLE
+^^^^^^^
+
+.. code-block:: python
+
+    # dangerous
+    with open(filename, 'w') as fp:
+        json.dump(data, fp)
+        # If an exception is raised, the file is empty or partly written
+
+    # safer
+    with safer.open(filename, 'w') as fp:
+        json.dump(data, fp)
+        # If an exception is raised, the file is unchanged.
+
+
+`safer.open(filename)` returns a file stream `fp` like `open(filename)`
+would, except that `fp` writes to memory stream or a temporary file in the
+same directory.
+
+If `fp` is used as a context manager and an exception is raised, then the
+property `fp.safer_failed` on the stream is automatically set to `True`.
+
+And when `fp.close()` is called, the cached data is stored in `filename` -
+*unless* `fp.safer_failed` is true.
+
+------------------------------------
+
+`safer.printer()`
 ~~~~~~~~~~~~~~~~~~~
 
-``safer.printer()`` is similar to ``safer.open()`` except it yields a function
+`safer.printer()` is similar to `safer.open()` except it yields a function
 that prints to the open file - it's very convenient for printing text.
 
-Like ``safer.open()``, if an exception is raised within its context manager,
+Like `safer.open()`, if an exception is raised within its context manager,
 the original file is left unchanged.
 
 EXAMPLE
@@ -150,14 +198,15 @@ from pathlib import Path
 import contextlib
 import functools
 import io
+import json
 import os
 import shutil
 import sys
 import tempfile
 import traceback
 
-__version__ = '4.1.2'
-__all__ = 'writer', 'open', 'closer', 'printer'
+__version__ = '4.4.1'
+__all__ = 'writer', 'open', 'closer', 'dump', 'printer'
 
 
 def writer(
@@ -167,27 +216,30 @@ def writer(
     temp_file=False,
     chunk_size=0x100000,
     delete_failures=True,
+    dry_run=False,
 ):
     """
     Write safely to file streams, sockets and callables.
 
-    ``safer.writer`` yields an in-memory stream that you can write
+    `safer.writer` yields an in-memory stream that you can write
     to, but which is only written to the original stream if the
     context finishes without raising an exception.
 
-    Because the actual writing happens when the context exits, it's possible
-    to block indefinitely if the underlying socket, stream or callable does.
+    Because the actual writing happens at the end, it's possible to block
+    indefinitely when the context exits if the underlying socket, stream or
+    callable does!
 
     ARGUMENTS
       stream:
         A file stream, a socket, or a callable that will receive data.
-        If stream is None, output is written to stdout
-        If stream is a string, this file is opened for writing.
+        If stream is `None`, output is written to `sys.stdout`
+        If stream is a string or `Path`, the file with that name is opened for
+        writing.
 
       is_binary:
-        Is ``stream`` a binary stream?
+        Is `stream` a binary stream?
 
-        If ``is_binary`` is ``None``, deduce whether it's a binary file from
+        If `is_binary` is ``None``, deduce whether it's a binary file from
         the stream, or assume it's text otherwise.
 
       close_on_exit: If True, the underlying stream is closed when the writer
@@ -207,19 +259,37 @@ def writer(
       delete_failures:
         If set to false, any temporary files created are not deleted
         if there is an exception
+
+      dry_run:
+        If dry_run is truthy, the stream is not written to at all at the end.
+
+        If dry_run is callable, the results of the stream are called with that
+        function rather than writing it to the underlying stream.
     """
     if isinstance(stream, (str, Path)):
         mode = 'wb' if is_binary else 'w'
-        return open(stream, mode, delete_failures=delete_failures)
+        return open(
+            stream, mode, delete_failures=delete_failures, dry_run=dry_run
+        )
 
     stream = stream or sys.stdout
-    if close_on_exit and stream in (sys.stdout, sys.stderr):
-        raise ValueError('You cannot close stdout or stderr')
 
-    write = getattr(stream, 'write', None)
+    if callable(dry_run):
+        write, dry_run = dry_run, True
+    elif dry_run:
+        write = len
+    else:
+        write = getattr(stream, 'write', None)
+
     send = getattr(stream, 'send', None)
     mode = getattr(stream, 'mode', None)
 
+    if dry_run:
+        close_on_exit = False
+
+    if close_on_exit and stream in (sys.stdout, sys.stderr):
+        raise ValueError('You cannot close stdout or stderr')
+
     if write and mode:
         if not set('w+a').intersection(mode):
             raise ValueError('Stream mode "%s" is not a write mode' % mode)
@@ -230,6 +300,9 @@ def writer(
 
         is_binary = binary_mode
 
+    elif dry_run:
+        pass
+
     elif send and hasattr(stream, 'recv'):  # It looks like a socket:
         if not (is_binary is None or is_binary is True):
             raise ValueError('is_binary=False is inconsistent with a socket')
@@ -273,28 +346,29 @@ def open(
     make_parents=False,
     delete_failures=True,
     temp_file=False,
+    dry_run=False,
 ):
     """
-    A drop-in replacement for ``open()`` which returns a stream which only
+    A drop-in replacement for `open()` which returns a stream which only
     overwrites the original file when close() is called, and only if there was
     no failure.
 
-    If a stream ``fp`` return from ``safer.open()`` is used as a context
-    manager and an exception is raised, the property ``fp.safer_failed`` is
-    set to ``True``.
+    If a stream `fp` return from `safer.open()` is used as a context
+    manager and an exception is raised, the property `fp.safer_failed` is
+    set to `True`.
 
-    In the method ``fp.close()``, if ``fp.safer_failed`` is *not* set, then the
+    In the method `fp.close()`, if `fp.safer_failed` is *not* set, then the
     cached results replace the original file, successfully completing the
     write.
 
-    If ``fp.safer_failed`` is true, then if ``delete_failures`` is true, the
+    If `fp.safer_failed` is true, then if `delete_failures` is true, the
     temporary file is deleted.
 
-    If the ``mode`` argument contains either ``'a'`` (append), or ``'+'``
+    If the `mode` argument contains either `'a'` (append), or `'+'`
     (update), then the original file will be copied to the temporary file
     before writing starts.
 
-    Note that if the ``temp_file`` argument is set, ``safer`` uses an extra
+    Note that if the `temp_file` argument is set, `safer` uses an extra
     temporary file which is renamed over the file only after the stream closes
     without failing. This uses as much disk space as the old and new files put
     together.
@@ -314,7 +388,10 @@ def open(
         directory as the target file, or in the system tempfile for streams
         that aren't files.
 
-    The remaining arguments are the same as for built-in ``open()``.
+      dry_run:
+         If dry_run is True, the file is not written to at all
+
+    The remaining arguments are the same as for built-in `open()`.
     """
     is_copy = '+' in mode or 'a' in mode
     is_read = 'r' in mode and not is_copy
@@ -340,6 +417,10 @@ def open(
     def simple_open():
         return __builtins__['open'](name, mode, buffering, **kwargs)
 
+    def simple_write(value):
+        with simple_open() as fp:
+            fp.write(value)
+
     if is_read:
         return simple_open()
 
@@ -347,9 +428,10 @@ def open(
         if '+' in mode:
             raise ValueError('+ mode requires a temp_file argument')
 
-        def write(value):
-            with simple_open() as fp:
-                fp.write(value)
+        if callable(dry_run):
+            write = dry_run
+        else:
+            write = len if dry_run else simple_write
 
         fp = _MemoryStreamCloser(write, True, is_binary).fp
         fp.mode = mode
@@ -374,7 +456,9 @@ def open(
     if buffering == -1:
         buffering = io.DEFAULT_BUFFER_SIZE
 
-    closer = _FileRenameCloser(name, temp_file, delete_failures, parent)
+    closer = _FileRenameCloser(
+        name, temp_file, delete_failures, parent, dry_run
+    )
 
     if is_copy and os.path.exists(name):
         shutil.copy2(name, closer.temp_file)
@@ -384,14 +468,78 @@ def open(
 
 def closer(stream, is_binary=None, close_on_exit=True, **kwds):
     """
-    Like ``safer.writer()`` but with ``close_on_exit=True`` by default
+    Like `safer.writer()` but with `close_on_exit=True` by default
 
     ARGUMENTS
-      Same as for ``safer.writer()``
+      Same as for `safer.writer()`
     """
     return writer(stream, is_binary, close_on_exit, **kwds)
 
 
+def dump(obj, stream=None, dump=None, **kwargs):
+    """
+    Safely serialize `obj` as a formatted stream to `fp`` (a
+    `.write()`-supporting file-like object, or a filename),
+    using `json.dump` by default
+
+    ARGUMENTS
+      obj:
+        The object to be serialized
+
+      stream:
+        A file stream, a socket, or a callable that will receive data.
+        If stream is `None`, output is written to `sys.stdout`.
+        If stream is a string or `Path`, the file with that name is opened for
+        writing.
+
+      dump:
+        A function or module or the name of a function or module to dump data.
+        If `None`, default to `json.dump``.
+
+      kwargs:
+        Additional arguments to `dump`.
+    """
+    if isinstance(stream, str):
+        name = stream
+        is_binary = False
+    else:
+        name = getattr(stream, 'name', None)
+        mode = getattr(stream, 'mode', None)
+        if name and mode:
+            is_binary = 'b' in mode
+        else:
+            is_binary = hasattr(stream, 'recv') and hasattr(stream, 'send')
+
+    if name and not dump:
+        dump = Path(name).suffix[1:] or None
+        if dump == 'yml':
+            dump = 'yaml'
+
+    if isinstance(dump, str):
+        try:
+            dump = __import__(dump)
+        except ImportError:
+            if '.' not in dump:
+                raise
+            mod, name = dump.rsplit('.', maxsplit=1)
+            dump = getattr(__import__(mod), name)
+
+    if dump is None:
+        dump = json.dump
+
+    elif not callable(dump):
+        try:
+            dump = dump.safe_dump
+        except AttributeError:
+            dump = dump.dump
+
+    with writer(stream) as fp:
+        if is_binary:
+            write = fp.write
+            fp.write = lambda s: write(s.encode('utf-8'))
+        return dump(obj, fp)
+
+
 @contextlib.contextmanager
 def printer(name, mode='w', *args, **kwargs):
     """
@@ -400,7 +548,7 @@ def printer(name, mode='w', *args, **kwargs):
     and only if there was no exception thrown
 
     ARGUMENTS
-      Same as for ``safer.open()``
+      Same as for `safer.open()`
     """
     if 'r' in mode and '+' not in mode:
         raise IOError('File not open for writing')
@@ -422,7 +570,6 @@ class _Closer:
             except Exception:
                 traceback.print_exc()
             raise
-
         self._close(self.fp.safer_failed)
 
     def _close(self, failed):
@@ -440,7 +587,7 @@ class _Closer:
     def _wrap(self, stream_cls):
         @functools.wraps(stream_cls)
         def wrapped(*args, **kwargs):
-            wrapped_cls = self._wrap_class(stream_cls)
+            wrapped_cls = _wrap_class(stream_cls)
             self.fp = wrapped_cls(*args, **kwargs)
             self.fp.safer_closer = self
             self.fp.safer_failed = False
@@ -448,22 +595,21 @@ class _Closer:
 
         return wrapped
 
-    @staticmethod
-    @functools.lru_cache()
-    def _wrap_class(stream_cls):
-        def members():
-            @functools.wraps(stream_cls.__exit__)
-            def __exit__(self, *args):
-                self.safer_failed = bool(args[0])
-                return stream_cls.__exit__(self, *args)
 
-            @functools.wraps(stream_cls.close)
-            def close(self):
-                self.safer_closer.close(stream_cls.close)
+# Wrap an existing IO class so that it calls safer at the end
+@functools.lru_cache()
+def _wrap_class(stream_cls):
+    @functools.wraps(stream_cls.__exit__)
+    def exit(self, *args):
+        self.safer_failed = bool(args[0])
+        return stream_cls.__exit__(self, *args)
 
-            return locals()
+    @functools.wraps(stream_cls.close)
+    def close(self):
+        self.safer_closer.close(stream_cls.close)
 
-        return type('Safer' + stream_cls.__name__, (stream_cls,), members())
+    members = {'__exit__': exit, 'close': close}
+    return type('Safer' + stream_cls.__name__, (stream_cls,), members)
 
 
 class _FileCloser(_Closer):
@@ -509,16 +655,25 @@ class _FileCloser(_Closer):
 
 
 class _FileRenameCloser(_FileCloser):
-    def __init__(self, target_file, temp_file, delete_failures, parent=None):
+    def __init__(
+        self,
+        target_file,
+        temp_file,
+        delete_failures,
+        parent=None,
+        dry_run=False,
+    ):
         self.target_file = target_file
+        self.dry_run = dry_run
         super().__init__(temp_file, delete_failures, parent)
 
     def _success(self):
-        if os.path.exists(self.target_file):
-            shutil.copymode(self.target_file, self.temp_file)
-        else:
-            os.chmod(self.temp_file, 0o100644)
-        os.rename(self.temp_file, self.target_file)
+        if not self.dry_run:
+            if os.path.exists(self.target_file):
+                shutil.copymode(self.target_file, self.temp_file)
+            else:
+                os.chmod(self.temp_file, 0o100644)
+            os.rename(self.temp_file, self.target_file)
 
 
 class _StreamCloser(_Closer):
